import oauth
import json
import sys
import dsutil
import argparse
import processor

def upsert(dataset, block, namespace=None):
    for ent in block:
        ent['key']['partitionId'] = {
            'projectId': dataset,
            'namespaceId': namespace
        }
    params = {
        'mode': 'NON_TRANSACTIONAL',
        'mutations': map(lambda ent: {'upsert': ent}, block)
    }
    return oauth.oauth_async_req_json('POST',
        'https://datastore.googleapis.com/v1/projects/%s:commit' % (dataset),
        params)

def remove(dataset, block, namespace=None):
    keys = []
    for ent in block:
        key = ent['key'].copy()
        key['partitionId'] = {
            'projectId': dataset,
            'namespaceId': namespace
        }
        keys.append(key)
    params = {
        'mode': 'NON_TRANSACTIONAL',
        'mutations': map(lambda key: {'delete': key}, keys)
    }
    return oauth.oauth_async_req_json('POST',
        'https://datastore.googleapis.com/v1/projects/%s:commit' % (dataset),
        params)

class BatchProcessor(processor.Processor):

    def __init__(self, dataset, kinds, namespace, operation, block_size = 500, parallel = 10):
        super(BatchProcessor, self).__init__(kinds, block_size)
        self.dataset = dataset
        self.namespace = namespace
        self.parallel = parallel
        self.ups = []
        self.operation = operation

    def consume(self, n):
        while len(self.ups) > n:
            print json.dumps(self.ups.pop(0).resp())

    def resolve(self):
        print >> sys.stderr, self.operation.__name__, self.processed
        self.ups.append(self.operation(self.dataset, self.block, self.namespace))
        self.consume(self.parallel)

    def done(self):
        self.consume(0)

def argparse_prepare(sub):
    sub.add_argument('-d', '--dataset', required=True, help='dataset')
    sub.add_argument('-n', '--namespace', help='namespace')
    sub.add_argument('-k', '--kinds', nargs='+', help='kinds')
    sub.add_argument('-b', '--block', type=int, help='block size')
    sub.add_argument('-p', '--parallel', type=int, help='parallel')
    sub.add_argument('-o', '--operation', required=True, choices=('upsert', 'remove'), help='operation')

def argparse_exec(args):
    op = upsert
    if args.operation == 'remove':
        op = remove
    processor = BatchProcessor(args.dataset, args.kinds, args.namespace, op, block_size = args.block or 500, parallel = args.parallel or 10)
    processor.process()



import reader
import argparse
import sys

class Error(Exception):
	"""Exceptions"""

def load_kinds(dataset, namespace=None):
	max_kinds = 1000
	results = reader.query(dataset, 'select __key__ from __kind__ order by __key__', namespace = namespace, limit = max_kinds)
	l = len(results['entities'])
	if l >= max_kinds:
		raise Error('Too many kinds: %i' % (max_kinds))
	ret = []
	for ent in results['entities']:
		path = ent['key']['path']
		name = path[0]['name']
		if not name.startswith('_'):
			ret.append(name)
	return ret

def print_iterate(dataset, kinds=[], namespace=None, keys_only=False):
	if not kinds:
		kinds = load_kinds(dataset, namespace)
	field = '*'
	if keys_only:
		field = '__key__'
	for kind in kinds:
		reader.print_iterate(dataset, 
			'select %s from `%s` order by __key__' % (field, kind), 
			namespace=namespace, msg=kind)

def argparse_prepare(sub):
	sub.add_argument('-d', '--dataset', required=True, help='dataset')
	sub.add_argument('-n', '--namespace', help='namespace')
	sub.add_argument('-k', '--kinds', nargs='+', help='kinds')
	sub.add_argument('-o', '--keys-only', help='keys only')

def argparse_exec(args):
	print_iterate(args.dataset, args.kinds, args.namespace, args.keys_only == 'true')




import httplib
import json
import util
import threading
from urlparse import urlparse

class Error(Exception):
	"""Exceptions"""

class HttpError(Error):
	"""Exception"""

def req_json(method, url, params = '', headers = {}, expects = [200]):
	parsed = urlparse(url)
	host = parsed.netloc
	uri = parsed.path
	if(parsed.query != ''):
		uri = uri + '?' + parsed.query
	conn = None
	if(parsed.scheme == 'https'):
		conn = httplib.HTTPSConnection(parsed.hostname, parsed.port or 443)
	else:
		conn = httplib.HTTPConnection(parsed.hostname, parsed.port or 80)
	if parsed.username != None:
		token = parsed.username + ':' + (parsed.password or '')
		token = 'Basic ' + base64.b64encode(token)
		headers['Authorization'] = token
	success = False
	try:
		conn.request(method, uri, params, headers)
		response = conn.getresponse()
		if response.status not in expects:
			raise Error('Status: %d %s %sri' % (response.status, response.reason, response.read()))
		string = response.read()
		if not string:
			return None
		ret = json.loads(string)
		success = True
		return ret
	finally:
		util.close(conn)

class HttpThread(threading.Thread):

	def __init__(self, method, url, params, headers, expects):
		threading.Thread.__init__(self)
		self.method = method
		self.url = url
		self.params = params
		self.headers = headers
		self.expects = expects
		self.response = None
		self.status = 'CREATED'

	def run(self):
		try:
			self.status = 'REQUESTING'
			self.response = req_json(self.method, self.url, self.params, self.headers, self.expects)
			self.status = 'DONE'
		finally:
			if self.status != 'DONE':
				self.status = 'ERROR'

	def resp(self):
		while self.status != 'DONE' and self.status != 'ERROR':
			self.join()
		if self.status == 'DONE':
			return self.response
		raise Error('Error on http thread, status: %s' % (self.status))

def async_req_json(method, url, params = '', headers = {}, expects = [200]):
	thread = HttpThread(method, url, params, headers, expects)
	thread.start()
	return thread

def __main():
	obj = async_req_json('GET', 'https://api.github.com/users/murer', headers = {
		'User-Agent': 'dsopz'
	}, expects = [200]).resp()
	print json.dumps(obj, indent=True)

if __name__ == '__main__':
	__main()



import json
import util
import os

def auth_file():
	directory = os.path.expanduser('~')
	if directory == '/':
		directory = '.'
	return directory + '/.dsopz/auth.json'

def delete_file():
	try:
		os.remove(auth_file())
	except OSError:
		pass

def write_file(content):
	c = json.dumps(content, indent=True)
	name = auth_file()
	util.makedirs(os.path.dirname(name))
	with open(name, 'w') as f:
		f.write(c + '\n')

def read_file():
	if not os.path.isfile(auth_file()):
		return None 
	with open(auth_file(), 'r') as f:
		c = f.read()
	return json.loads(c)



import argparse
import processor
import sys
import json

class IndexedProcessor(processor.Processor):

    def __init__(self, columns, indexed, kinds = []):
        super(IndexedProcessor, self).__init__(kinds)
        self.columns = columns
        self.excludeFromIndexes = not indexed

    def __setExcludeFromIndexes(self, column, excludeFromIndexes):
        if column.get('excludeFromIndexes') == excludeFromIndexes:
            return False
        column['excludeFromIndexes'] = excludeFromIndexes
        return True

    def resolve(self):
        print >> sys.stderr, 'process', self.processed
        for ent in self.block:
            changed = False
            for name in self.columns:
                column = ent['properties'].get(name)
                if not column:
                    continue
                # ----- wtf is listValue? ----
                #if column.get('listValue'):
                #    for k in column['listValue']:
                #        if self.__setExcludeFromIndexes(k, self.excludeFromIndexes):
                #            changed = True
                if column.get('arrayValue'):
                    for k in column['arrayValue'].get('values') or []:
                        if self.__setExcludeFromIndexes(k, self.excludeFromIndexes):
                            changed = True
                else:
                    if self.__setExcludeFromIndexes(column, self.excludeFromIndexes):
                        changed = True
            if changed:
                print json.dumps(ent)

def argparse_prepare(sub):
    sub.add_argument('-k', '--kinds', nargs='+', help='kinds')
    sub.add_argument('-c', '--columns', required=True, nargs='+', help='columns')
    sub.add_argument('-i', '--indexed', required=True, choices=('false', 'true'), help='indexed')

def argparse_exec(args):
    processor = IndexedProcessor(args.columns, args.indexed == 'true', args.kinds)
    processor.process()



import argparse
import dsutil
import sys
import json

class Error(Exception):
	"""Exceptions"""

class ProcessorError(Error):
	"""Exceptions"""

class Processor(object):

	def __init__(self, kinds = [], block_size=500):
		self.kinds = kinds or []
		self.kinds = [k.lower() for k in self.kinds]
		self.block_size = block_size

	def resolve(self):
		raise ProcessorError('You need to implement process')

	def done(self):
		""" Done """

	def process(self):
		self.block = []
		self.processed = 0
		while True:
			line = sys.stdin.readline()
			if not line:
				break
			line = line.strip()
			if not line or line.startswith('#'):
				continue
			obj = json.loads(line)
			kind = dsutil.get_kind(obj)
			if self.kinds and kind.lower() not in self.kinds:
				continue
			self.block.append(obj)
			if len(self.block) >= self.block_size:
				self.processed += len(self.block)
				self.resolve()
				self.block = []
		if self.block:
			self.processed += len(self.block)
			self.resolve()
			self.block = []
		self.done()

class PrintProcessor(Processor):

	def __init__(self, columns, kinds = [], separator = '\t'):
		super(PrintProcessor, self).__init__(kinds)
		self.columns = columns
		if separator == None:
			self.separator = '\t'
		else:
			self.separator = separator.decode('string_escape')

	def resolve(self):
		for ent in self.block:
			line = ''
			for i, column in enumerate(self.columns):
				value, t = dsutil.prop_value(ent, column)
				if column == '__key__' or t == 'keyValue':
					value = dsutil.human_key(value)
				else:
					value = json.dumps(value)
				line += value
				if i < len(self.columns) - 1:
					line += self.separator
			print line

	def headers(self):
		line = ''
		for i, column in enumerate(self.columns):
			line += column
			if i < len(self.columns) - 1:
				line += self.separator
		print line

def __main():
	parser = argparse.ArgumentParser(description='Processor')
	parser.add_argument('-k', '--kinds', nargs='+', help='kinds')
	parser.add_argument('-c', '--columns', required=True, nargs='+', help='columns')
	parser.add_argument('-s', '--separator', help='separator')
	parser.add_argument('-a', '--headers', help='headers')
	args = parser.parse_args()
	processor = PrintProcessor(args.columns, args.kinds, args.separator)
	if args.headers:
		processor.headers()
	processor.process()

if __name__ == '__main__':
	__main()



import oauth_local
import oauth_installed
import oauth_gce
import oauth_serviceaccount
import sys
import time
import http
import json
import oauth_base

class Error(Exception):
	"""Exceptions"""

def resolve(t):
	if t == 'installed':
		return oauth_installed
	elif t == 'local':
		return oauth_local
	elif t == 'gce':
		return oauth_gce
	elif t == 'serviceaccount':
		return oauth_serviceaccount
	raise Error('Unknown: %s' % (t))

def get_token():
	auth = oauth_base.read_file()
	if not auth:
		raise Error('You need to login')
	now = now = int(time.time())
	handler = resolve(auth['handler'])
	if now > auth['expires'] - 60:
		handler.refresh_token(auth)
	auth = oauth_base.read_file()
	if not auth:
		raise Error('You need to login')
	return auth['access_token']

def oauth_req_json(method, url, params = None, headers = {}, expects = [200]):
	return oauth_async_req_json(method, url, params, headers, expects).resp()

def oauth_async_req_json(method, url, params = None, headers = {}, expects = [200]):
	headers['Authorization'] = 'Bearer %s' % get_token()
	if params:
		params = json.dumps(params)
		headers['Content-type'] = 'application/json; charset=UTF-8'
	return http.async_req_json(method, url, params, headers, expects)


def argparse_prepare(sub):
	""" ok """

def argparse_exec(args):
	print get_token()

def __main():
	print get_token()

if __name__ == '__main__':
	__main()



import argparse
import processor
import sys
import json
import os

def emit(*ents):
	for ent in ents:
		if type(ent) != type([]):
			ent = [ent]
		for k in ent:
			print json.dumps(k)

class ProcessorMapper(processor.Processor):

	def __init__(self, mapper, kinds = []):
		super(ProcessorMapper, self).__init__(kinds)
		self.mapper = mapper

	def resolve(self):
		print >> sys.stderr, 'process', self.processed
		for ent in self.block:
			exec(self.mapper)

def argparse_prepare(sub):
	sub.add_argument('-k', '--kinds', nargs='+', help='kinds')

def argparse_exec(args):
	try:
		with os.fdopen(3, 'r') as f:
			mapper = f.read()
	except OSError as e:
		print >> sys.stderr, 'Try 3<map.py'
		return sys.exit(1)
	processor = ProcessorMapper(mapper, args.kinds)
	processor.process()





import argparse
import processor
import sys
import json

class UpdateProcessor(processor.Processor):

    def __init__(self, props, kinds = []):
        super(UpdateProcessor, self).__init__(kinds)
        self.props = json.loads(props)

    def resolve(self):
        print >> sys.stderr, 'process', self.processed
        for ent in self.block:
            for name,value in self.props.iteritems():
                if value.get('excludeFromIndexes') == None:
                    value['excludeFromIndexes'] = True

                ent['properties'][name] = value
            print json.dumps(ent)

def argparse_prepare(sub):
    sub.add_argument('-k', '--kinds', nargs='+', help='kinds')
    sub.add_argument('-p', '--properties', required=True,  help='properties')

def argparse_exec(args):
    processor = UpdateProcessor(args.properties ,args.kinds)
    processor.process()



import json

def get_kind(obj):
	path = obj['key']['path']
	i = len(path)
	last = path[i - 1]
	return last['kind']

def prop(entity, name):
	if name == '__key__':
		return entity['key']['path']
	return entity['properties'].get(name)

def prop_value(entity, name):
	p = prop(entity, name)
	if not p or name == '__key__':
		return p, None
	for k, v in p.iteritems():
		if k.endswith('Value'):
			return v, k
	return None, None

def human_key(key):
	if not key:
		return 'null'
	if type(key) == type(dict()):
		key = key.get('path')
	if not key:
		return 'null'
	ret = ''
	for k in key:
		ret += ('%s/%s' % (k['kind'], json.dumps(k.get('name') or k.get('id'))))
	return ret




import argparse
import importer
import exporter
import reader
import console
import processor_indexed
import processor_csv
import processor_sql
import processor_update
import processor_mapper
import oauth
import oauth_gce
import oauth_installed
import oauth_local
import oauth_serviceaccount
import config

class Version(object):

    def argparse_prepare(self, sub):
        """ ok """

    def argparse_exec(self, args):
        print(config.version)

class Parser(object):

    def __init__(self):
        self.parser = argparse.ArgumentParser(description='DSOpz')
        self.subparsers = self.parser.add_subparsers(dest="subparsers", title="Command")
        self.mods = {}

    def prepare(self, name, mod):
        sub = self.subparsers.add_parser(name)
        mod.argparse_prepare(sub)
        self.mods[name] = mod

    def handle_args(self):
        args = self.parser.parse_args()
        mod = self.mods[args.subparsers]
        mod.argparse_exec(args)

def main():
    parser = Parser()
    parser.prepare('version', Version())
    parser.prepare('console', console)
    parser.prepare('export', exporter)
    parser.prepare('import', importer)
    parser.prepare('gql', reader)
    parser.prepare('index', processor_indexed)
    parser.prepare('csv', processor_csv)
    parser.prepare('sql', processor_sql)
    parser.prepare('update', processor_update)
    parser.prepare('map', processor_mapper)
    parser.prepare('token', oauth)
    parser.prepare('login', oauth_local)
    parser.prepare('login-text', oauth_installed)
    parser.prepare('login-gce', oauth_gce)
    parser.prepare('login-serviceaccount', oauth_serviceaccount)
    parser.handle_args()

if __name__ == '__main__':
    main()



import argparse
import processor
import csv
import sys
import dsutil
import locale
import sqlite3

class SQLProcessor(processor.Processor):

	def __init__(self, columns, kinds = []):
		super(SQLProcessor, self).__init__(kinds)
		self.columns = columns
		self.sql = 'insert into %s (';
		self.sql += ', '.join(['id' if str(x) == '__key__' else str(x) for x in columns])
		self.sql += ') values ('
		self.sql += ', '.join(['%s' for x in columns])
		self.sql += ');'

	def resolve(self):
		print >> sys.stderr, 'process', self.processed
		for ent in self.block:
			kind = dsutil.get_kind(ent)
			line = [ ]
			for column in self.columns:
				name  = column
				value, t = dsutil.prop_value(ent, column)
				if column == '__key__':
					name = 'id'
					last = value[len(value) - 1]
					value = last.get('name') or last.get('id')
				if value:
					value = value.encode('UTF-8')
				line.append(value)
			values = [ ("'%s'" % (str(x).replace("'", "''"))) for x in line]
			values = [ kind ] + values
			print self.sql % tuple(values)

def argparse_prepare(sub):
	sub.add_argument('-k', '--kinds', nargs='+', help='kinds')
	sub.add_argument('-c', '--columns', required=True, nargs='+', help='columns')

def argparse_exec(args):
	processor = SQLProcessor(args.columns, args.kinds)
	processor.process()



import oauth
import json
import sys
import argparse

def __parse_entity_results(resp):
    ents = resp['batch'].get('entityResults') or []
    ret = []
    for ent in ents:
        ent = ent['entity']
        copy = {
            'key': {
                'path': ent['key']['path']
            },
            'properties': {}
        }
        for k, v in ent.get('properties', {}).iteritems():
            prop = v.copy()
            excludeFromIndexes = prop.pop('excludeFromIndexes', False)
            prop['excludeFromIndexes'] = excludeFromIndexes
            # prop['excludeFromIndexes'] = prop.pop('excludeFromIndexes', False)
            # if prop.get('listValue'):
            #     prop.pop('indexed')
            if v:
                copy['properties'][k] = prop
        ret.append(copy)
    return ret

def query(dataset, gql, namespace=None, limit=1000, startCursor=None):
    url = 'https://datastore.googleapis.com/v1/projects/%s:runQuery' % (dataset)
    queryString = gql
    if limit:
        queryString = '%s limit %i' % (gql, limit)
    if startCursor:
        queryString += ' offset @startCursor'
    params = {
        'partitionId': {
            'projectId': dataset,
            'namespaceId': namespace
        },
        'gqlQuery': {
            'allowLiterals': True,
            'queryString': queryString
        }
    }
    if startCursor:
        params['gqlQuery']['namedBindings'] = { 'startCursor': { 'cursor': startCursor } };
    resp = oauth.oauth_req_json('POST', url, params)
    ret = {}
    ret['entities'] = __parse_entity_results(resp)
    ret['endCursor'] = resp['batch'].get('endCursor')
    return ret

def iterate(dataset, gql, namespace=None, bulkSize=1000):
    startCursor = None
    while True:
        page = query(dataset, gql, namespace, bulkSize, startCursor)
        if not page['entities']:
            return
        startCursor = page.get('endCursor')
        for ent in page['entities']:
            yield ent


def print_iterate(dataset, gql, namespace=None, msg=''):
    it = iterate(dataset, gql, namespace)
    loaded = 0
    try:
        while True:
            loaded += 1
            if loaded % 1000 == 0:
                print >> sys.stderr, 'loaded', msg, loaded
            string = json.dumps(it.next(), sort_keys=True)
            print string
    except StopIteration:
        pass
    print >> sys.stderr, 'Done', msg, loaded-1

def argparse_prepare(sub):
    sub.add_argument('-d', '--dataset', required=True, help='dataset')
    sub.add_argument('-n', '--namespace', help='namespace')
    sub.add_argument('-q', '--gql', required=True, help='gql')

def argparse_exec(args):
    print_iterate(args.dataset, args.gql, args.namespace)



import os
import errno
import json

def close(obj):
	try:
		obj.close()
	except:
		""" Done """

def makedirs(directory):
	try:
		os.makedirs(directory)
	except OSError as exc:
		if exc.errno == errno.EEXIST and os.path.isdir(directory):
			pass
		else:
			raise




from distutils.core import setup
import dsopz.config as config

setup(
  name = 'dsopz',
  packages = ['dsopz'],
  version = config.version,
  description = 'Google Datastore Operations',
  entry_points = {
      "console_scripts": [
          "dsopz = dsopz:main",
      ]
  },
  author = 'Paulo Henrique Murer',
  author_email = 'fuweweu@gmail.com',
  url = 'https://github.com/murer/dsopz', 
  download_url = 'https://github.com/murer/dsopz/tarball/dsopz-%s' % (config.version),
  keywords = ['gae', 'datastore', 'appengine'],
  classifiers = [],
)






import http
import urllib
import time
import util
import sys
import oauth_base

class Error(Exception):
	"""Exceptions"""

def __config():
	return {
		'client_id': '570403801115-fik4r8kkcf89d7c46mepm5keekker8jl.apps.googleusercontent.com',
		'client_secret': 'luZsJXmEfBr4iP0WoruMbZz1',
		'scopes': [
			'https://www.googleapis.com/auth/cloud-platform',
			'https://www.googleapis.com/auth/datastore',
			'https://www.googleapis.com/auth/userinfo.email'
		]
	}

def login():
	oauth_base.delete_file()
	config = __config()
	url = 'https://accounts.google.com/o/oauth2/auth?' + urllib.urlencode({
		'client_id': config['client_id'],
		'redirect_uri': 'urn:ietf:wg:oauth:2.0:oob',
		'response_type': 'code',
		'scope': ' '.join(config['scopes']),
		'approval_prompt': 'force',
		'access_type': 'offline'
	})
	print 'Browser:'
	print url
	print 'Code:'
	code = sys.stdin.readline().strip()
	content = http.req_json('POST', 'https://www.googleapis.com:443/oauth2/v3/token', urllib.urlencode({
		'code': code,
		'client_id': config['client_id'],
		'client_secret': config['client_secret'],
		'redirect_uri': 'urn:ietf:wg:oauth:2.0:oob',
		'grant_type': 'authorization_code'
	}), { 'Content-Type': 'application/x-www-form-urlencoded;charset=UTF-8' })
	now = now = int(time.time())
	expires_in = content['expires_in']
	content['created'] = now
	content['expires'] = now + expires_in
	content['handler'] = 'installed'
	oauth_base.write_file(content)
	print 'Logged in'

def refresh_token(auth):
	config = __config()
	content = http.req_json('POST', 'https://www.googleapis.com/oauth2/v3/token', urllib.urlencode({
		'refresh_token': auth['refresh_token'],
		'client_id': config['client_id'],
		'client_secret': config['client_secret'],
		'grant_type': 'refresh_token'
	}), { 'Content-Type': 'application/x-www-form-urlencoded;charset=UTF-8' })
	now = now = int(time.time())
	expires_in = content['expires_in']
	content['created'] = now
	content['expires'] = now + expires_in
	content['refresh_token'] = auth['refresh_token']
	content['handler'] = 'installed'
	oauth_base.write_file(content)

def argparse_prepare(sub):
	""" ok """

def argparse_exec(args):
	login()



import http
import time
import util
import oauth_base

class Error(Exception):
	"""Exceptions"""

def login():
	oauth_base.delete_file()
	refresh_token(None)

def refresh_token(auth):
	content = http.req_json('GET',
		'http://metadata.google.internal/computeMetadata/v1/instance/service-accounts/default/token',
		headers = { 'Metadata-Flavor': 'Google' })
	now = now = int(time.time())
	expires_in = content['expires_in']
	content['created'] = now
	content['expires'] = now + expires_in
	content['handler'] = 'gce'
	oauth_base.write_file(content)
	print 'Logged in'

def argparse_prepare(sub):
	""" ok """

def argparse_exec(args):
	login()



import argparse
import sys
import reader
import dsutil
import json
import cmd
import re
import http
import time
import sys

class Console(cmd.Cmd):

	def __init__(self, dataset, namespace=None, prompt=True,source=sys.stdin, 
			dest=sys.stdout, show_size=True, seperator='\t', double_endline=True,
			limit=None):
		cmd.Cmd.__init__(self, stdin=source, stdout=dest)
		self.dataset = dataset
		self.namespace = namespace
		self.prompt = ''
		if prompt:
			self.prompt = '> '
		self.show_size = show_size
		self.seperator = '\t'
		if seperator != None:
			self.seperator = seperator
		self.double_endline = double_endline
		self.limit = limit or 0

	def show_entities(self, gql, result, fields):
		for ent in result:
			key = dsutil.human_key(ent['key'])
			line = key
			if fields[0] == '*':
				fields = []
				for k in ent['properties'].iterkeys():
					fields.append(k)
			for p in fields:
				v, t = dsutil.prop_value(ent, p)
				prop = ent['properties'].get(p)
				line += self.seperator
				if prop:
					i = prop.get('indexed')
					t = t.replace('Value', '')
					line += '%s/%s/%s/%s' % (p, t, i, json.dumps(v))
			print >> self.stdout, line
			if self.double_endline:
				print >> self.stdout, ''

	def process(self, gql, fields):
		before = int(time.time())
		bulkSize = 1000
		if self.limit > 0:
			bulkSize = min(bulkSize, self.limit)
		it = reader.iterate(self.dataset, gql, namespace=self.namespace, bulkSize=bulkSize)
		loaded = 0
		limited = ''
		try:
			while True:
				ent = it.next()
				self.show_entities(gql, [ent], fields)
				loaded += 1
				if self.limit and loaded >= self.limit:
					limited = 'limited '
					break 
		except StopIteration:
			pass
		after = int(time.time())
		if self.show_size:
			print >> self.stdout, 'Total: %s %s(%s seconds)' % (loaded, limited, (after - before))

	def parse_gql(self, line):
		temp = re.sub(r'\sfrom\s.*$', '', line)
		temp = re.sub(r'\sgroup\s.*$', '', temp)
		temp = re.sub(r'\slimit\s.*$', '', temp)
		temp = re.sub(r'\soffset\s.*$', '', temp)
		temp = re.sub(r'\sorder\s.*$', '', temp)
		temp = re.sub(r'\swhere\s.*$', '', temp)
		line = line.replace(temp, 'select *')
		fields = re.split(r'[\s,]+', temp)
		return fields, line 

	def do_select(self, line):
		fields, gql = self.parse_gql(line)
		try:
			self.process(gql, fields)
		except KeyboardInterrupt:
			print >> self.stdout, 'Interrupted'
		except Exception, e:
			print >> self.stdout, 'Error', e

	def do_EOF(self, line):
		return self.do_exit('exit')

	def do_exit(self, line):
		return True

	def emptyline(self):
		""" ok """

	def default(self, line):
		if line:
			line = line.strip()
		if not line.startswith('#') and not line.startswith('--'):
			cmd.Cmd.default(self, line)

def argparse_prepare(sub):
	sub.add_argument('-d', '--dataset', required=True, help='dataset')
	sub.add_argument('-n', '--namespace', help='namespace')
	sub.add_argument('-p', '--no-prompt', dest='prompt', action='store_false', help='no prompt')
	sub.add_argument('-sl', '--single-line', dest='single_line', action='store_true', help='single line')
	sub.add_argument('-ns', '--no-summary', dest='no_summary', action='store_true', help='single line')
	sub.add_argument('-l', '--limit', help='limit', type=int)
	sub.add_argument('-s', '--seperator', help='seperator')

def argparse_exec(args):
	limit = args.limit
	if limit == None:
		limit = 10
	c = Console(args.dataset, namespace=args.namespace, prompt=args.prompt, 
		double_endline=not args.single_line, show_size=not args.no_summary,
		limit=limit, seperator=args.seperator)
	try:
		c.cmdloop()
	except KeyboardInterrupt:
		return




import argparse
import processor
import csv
import sys
import dsutil
import locale

class CSVProcessor(processor.Processor):

	def __init__(self, columns, kinds = []):
		super(CSVProcessor, self).__init__(kinds)
		self.columns = columns
		l = []
		for k in self.columns:
			if k == '__key__':
				k = 'key'
			l.append(k)
		self.writer = csv.DictWriter(sys.stdout, l ,delimiter=';', quoting=csv.QUOTE_ALL)
		self.writer.writeheader()

	def resolve(self):
		print >> sys.stderr, 'process', self.processed
		for ent in self.block:
			line = {}
			for column in self.columns:
				name  = column
				value, t = dsutil.prop_value(ent, column)
				if column == '__key__':
					name = 'key'
					last = value[len(value) - 1]
					value = last.get('name') or last.get('id')
				if value:
					value = value.encode('UTF-8')
				line[name] = value
			self.writer.writerow(line)

def argparse_prepare(sub):
	sub.add_argument('-k', '--kinds', nargs='+', help='kinds')
	sub.add_argument('-c', '--columns', required=True, nargs='+', help='columns')

def argparse_exec(args):
	processor = CSVProcessor(args.columns, args.kinds)
	processor.process()




import http
import time
import util
import oauth_base
import base64
import json
import urllib

class Error(Exception):
	"""Exceptions"""

def __config():
	return {
		'scopes': [
			'https://www.googleapis.com/auth/cloud-platform',
			'https://www.googleapis.com/auth/datastore',
			'https://www.googleapis.com/auth/userinfo.email'
		]
	}

def login(credential):
	oauth_base.delete_file()

	try:
		from Crypto.Hash import SHA256
		from Crypto.PublicKey import RSA
		from Crypto.Signature import PKCS1_v1_5
	except ImportError:
		raise Error('Service account authentication requires pycrypto')

	privatekey = RSA.importKey(credential['private_key'])
	config = __config()
	now = int(time.time())
	header = base64.urlsafe_b64encode(json.dumps({
		'alg': 'RS256',
		'typ': 'JWT',
		'kid': credential['private_key_id']
	}).encode('UTF-8'))
	payload = base64.urlsafe_b64encode(json.dumps({
		'iss': credential['client_email'],
		'scope': ' '.join(config['scopes']),
		'aud': credential['token_uri'],
		'exp': (now + 3600),
		'iat': now
	}).encode('UTF-8'))
	token = '%s.%s' % (header, payload)
	sign = token.encode('UTF-8')
	sign = PKCS1_v1_5.new(privatekey).sign(SHA256.new(sign))
	sign = base64.urlsafe_b64encode(sign)
	assertion = '%s.%s' % (token, sign)
	auth = {
		'handler': 'serviceaccount',
		'assertion': assertion,
		'token_uri': credential['token_uri']
	}
	refresh_token(auth)
	print 'Logged in'

def refresh_token(auth):
	content = http.req_json('POST', auth['token_uri'], urllib.urlencode({
		'grant_type': 'urn:ietf:params:oauth:grant-type:jwt-bearer',
		'assertion': auth['assertion']
	}), { 'Content-Type': 'application/x-www-form-urlencoded;charset=UTF-8' })
	now = int(time.time())
	expires_in = content['expires_in']
	content['created'] = now
	content['expires'] = now + expires_in
	content['assertion'] = auth['assertion']
	content['handler'] = 'serviceaccount'
	oauth_base.write_file(content)

def argparse_prepare(sub):
	sub.add_argument('-f', '--file', required=True, help='service account json file')

def argparse_exec(args):
	with open(args.file, 'r') as f:
		content = f.read()
	credential = json.loads(content)
	login(credential)
